from com.google.common.base import Function
from nodebox.graphics import Point

# Round a value to the nearest "step".
#
# v        - The value to snap.
# distance - The distance between steps.
# strength - The strength of rounding. If 1 the values will always be on a step. If zero, the value is unchanged.
#
# Returns the snapped value.
def _snap(v, distance, strength):
    strength = float(strength)
    return (v * (1.0 - strength)) + (strength * round(v / distance) * distance)

        #print "apply"
        #return point

# Snap the shape to a grid.
#
# shape    - The shape to snap.
# distance - The grid size, or distance between grid lines.
# strength - The snap strength, between 0.0-100.0. If 0.0, no snapping occurs. If 100.0, all points are on the grid.
# position - The grid position.
#
# Returns the snapped geometry.
def snap(shape, distance, strength, position):
    if shape is None: return None
    strength = strength / 100.0

    class Fn(Function):
        def __init__(self):
            print "Fn constructor"

        def apply(self, point):
            if point is None:
                return Point.ZERO
            x = _snap(point.x + position.x, distance, strength) - position.x
            y = _snap(point.y + position.y, distance, strength) - position.y
            return Point(x, y, point.type)

    return shape.mapPoints(Fn())

